@@ -122,7 +122,8 @@ C$OMP FLUSH(yout)
       return
       end
       subroutine awslchi2(s,ksi,ni,sigma,vpar,L,mask,n1,n2,n3,ind,
-     1      w,nw,minni,wad,sad,lambda,nthreds,iL,work,thn,sigman,ksin)
+     1      w,nw,minni,wad,sad,lambda,nthreds,iL,work,thn,sigman,ksin,
+     2      flb,nfb)
 C
 C  local variance estimation using (adaptive) weighted likelihood
 C
@@ -145,15 +146,15 @@ C   ind(.,i)[1:3] are j1-i1,j2-i2 and j3-i3 respectively
 C   wad, sad - array for weights>0 and corresponding observed s
 C
       implicit none
-      integer n1,n2,n3,nw,ind(3,nw),nthreds,iL
+      integer n1,n2,n3,nw,ind(3,nw),nthreds,iL,nfb
       integer mask(n1,n2,n3)
       double precision s(n1,n2,n3),ni(n1*n2*n3),thn(n1*n2*n3),
      1  ksi(n1,n2,n3),sigman(n1*n2*n3),lambda,w(nw),sigma(n1,n2,n3),
      2  wad(nw,nthreds),sad(nw,nthreds),L,minni,work(iL,nthreds),
-     3  ksin(n1,n2,n3),vpar(6)
+     3  ksin(n1,n2,n3),vpar(6),flb(nfb)
       integer i1,i2,i3,j1,j2,j3,i,j,jj,n,maxit,thrednr
       double precision z,sw,sws,sws2,sj,thi,wj,kval,fnsi,sgi,tol,low,up,
-     1       fmin,sgi2,vz,thi2,thj2,fnsj,thj
+     1       fmin,sgi2,vz,thi2,thj2,fnsj,thj,nii
 !$      integer omp_get_thread_num
 !$      external omp_get_thread_num
       n = n1*n2*n3
@@ -164,7 +165,7 @@ C  precompute values of lgamma(corrected df/2) in each voxel
 C$OMP PARALLEL DEFAULT(SHARED)
 C$OMP& FIRSTPRIVATE(iL,L,minni,n1,n2,n3,maxit)
 C$OMP& PRIVATE(i,j,i1,i2,i3,j1,j2,j3,z,sw,sws,sws2,thi,kval,thi2,thj2,
-C$OMP& wj,sj,thrednr,fnsi,low,up,tol,sgi,jj,fmin,sgi2,vz,thj,fnsj)
+C$OMP& wj,sj,thrednr,fnsi,low,up,tol,sgi,jj,fmin,sgi2,vz,thj,fnsj,nii)
 C$OMP DO SCHEDULE(GUIDED)
       DO i=1,n
          i1=mod(i,n1)
@@ -179,7 +180,7 @@ C$OMP DO SCHEDULE(GUIDED)
          sws2=0.d0
          sgi=sigma(i1,i2,i3)
          sgi2=sgi*sgi
-         thi = sqrt(max(0.d0,ksi(i1,i2,i3)/sgi2-2.d0*L))
+         thi = sqrt(max(1d-16,ksi(i1,i2,i3)/sgi2-2.d0*L))
          thn(i) = thi
          if(thi.gt.vpar(1)) THEN
             thi2 = thi*thi
@@ -188,8 +189,12 @@ C$OMP DO SCHEDULE(GUIDED)
          ELSE
             fnsi = vpar(2)
          END IF
+         nii = ni(i)
 C   thats the estimated standard deviation of s(i1,i2,i3)
-         kval = lambda/ni(i)
+         kval = lambda/nii*(sgi/nii+thi)/(0.1d0/nii*sgi+thi)
+C allow for increased kval for low SNR and small ni
+C correction vanishes for nii -> \infty and thi>0
+C without this the propagation condition is violated for very small SNR 
          jj = 0
          DO j=1,nw
             wad(j,thrednr)=0.d0
@@ -201,7 +206,7 @@ C   thats the estimated standard deviation of s(i1,i2,i3)
             if(j3.le.0.or.j3.gt.n3) CYCLE
             if(mask(j1,j2,j3).eq.0) CYCLE
             wj=w(j)
-            thj = sqrt(max(0.d0,ksi(j1,j2,j3)/sgi2-2.d0*L))
+            thj = sqrt(max(1d-16,ksi(j1,j2,j3)/sgi2-2.d0*L))
             if(thj.gt.vpar(1)) THEN
                thj2 = thj*thj
                vz = vpar(3)*thj+vpar(4)*thj2+vpar(5)*thj*thj2+vpar(6)
@@ -226,26 +231,117 @@ C   thats the estimated standard deviation of s(i1,i2,i3)
             ksin(i1,i2,i3) = sws2/sw
 C needed for the next iteration
 C            low = max(sqrt(ksin(i1,i2,i3)/2.d0/L),sgi/1d1)
-             low = sgi/1d1
+             low = sgi/2d0
 C  sqrt(ksin(i1,i2,i3)/2.d0/L) is the solution in the central case !
 C  old code was still correct but inefficient
 C            up = sgi*1d1
-            up = min(sqrt(ksin(i1,i2,i3)/2.d0/L),sgi*1d1)
+            up = min(sqrt(ksin(i1,i2,i3)/2.d0/L),sgi*2d0)
             if(up.le.low) THEN
                sgi = up
             ELSE
                call localmin(low,up,wad(1,thrednr),sad(1,thrednr),L,jj,
-     1                    tol,maxit,work(1,thrednr),sgi,fmin)
+     1                    tol,maxit,work(1,thrednr),sgi,fmin,flb,nfb)
             END IF
          END IF
          sigman(i)=sgi
-         thn(i) = sqrt(max(0.d0,ksin(i1,i2,i3)-2.d0*sgi*sgi*L))
+         thn(i) = sqrt(max(1.d-16,ksin(i1,i2,i3)-2.d0*sgi*sgi*L))
       END DO
 C$OMP END DO NOWAIT
 C$OMP END PARALLEL
 C$OMP FLUSH(thn,ni,sigman)
       RETURN
       END
+C
+C   same for Gaussian distribution
+C
+      subroutine awslgaus(s,th,ni,sigma,mask,n1,n2,n3,ind,
+     1      w,nw,minni,lambda,thn,sigman)
+C
+C  local variance estimation for Gaussian data
+C  using (adaptive) weighted likelihood
+C
+C   Takes observed intensities in s and
+C     initial estimates of \sigma in sigma
+C   perform adaptive smoothing on R^3
+C   th containes previous estimates of E S
+C   ni containes previous sum of weights
+C   mask - logical mask (use if mask==TRUE)
+C   n1,n2,n3 - dimensions
+C   ind  - integer array dim (3,n) containing relative indices in xyz
+C   w    - vector of corresponding location weights
+C   nw   - number of positive weights (initial value
+C
+C   lambda   - kritical value for pairwise tests
+C   thn      - new estimate sum_j w_a(j) S_j
+C   ind(.,i) contains coordinate indormation corresponding to positive
+C   location weights in w(i)
+C   ind(.,i)[1:3] are j1-i1,j2-i2 and j3-i3 respectively
+C   wad, sad - array for weights>0 and corresponding observed s
+C
+      implicit none
+      integer n1,n2,n3,nw,ind(3,nw)
+      integer mask(n1,n2,n3)
+      double precision s(n1,n2,n3),ni(n1*n2*n3),thn(n1*n2*n3),
+     1 th(n1,n2,n3),sigman(n1*n2*n3),lambda,w(nw),sigma(n1,n2,n3),minni
+      integer i1,i2,i3,j1,j2,j3,i,j,n,thrednr
+      double precision z,sw,sws,sws2,sj,thi,wj,kval,sgi
+!$      integer omp_get_thread_num
+!$      external omp_get_thread_num
+      n = n1*n2*n3
+      thrednr = 1
+C  precompute values of lgamma(corrected df/2) in each voxel
+C$OMP PARALLEL DEFAULT(SHARED)
+C$OMP& PRIVATE(i,j,i1,i2,i3,j1,j2,j3,z,sw,sws,sws2,thi,kval,
+C$OMP& wj,sj,thrednr,sgi)
+C$OMP DO SCHEDULE(GUIDED)
+      DO i=1,n
+         i1=mod(i,n1)
+         if(i1.eq.0) i1=n1
+         i2=mod((i-i1)/n1+1,n2)
+         if(i2.eq.0) i2=n2
+         i3=(i-i1-(i2-1)*n1)/n1/n2+1
+         if(mask(i1,i2,i3).eq.0) CYCLE
+!$         thrednr = omp_get_thread_num()+1
+         sw=0.d0
+         sws=0.d0
+         sws2=0.d0
+         sgi=sigma(i1,i2,i3)
+         thi = th(i1,i2,i3)
+C   thats the estimated standard deviation of s(i1,i2,i3)
+         kval = 2.d0*lambda/ni(i)
+         DO j=1,nw
+            j1=i1+ind(1,j)
+            if(j1.le.0.or.j1.gt.n1) CYCLE
+            j2=i2+ind(2,j)
+            if(j2.le.0.or.j2.gt.n2) CYCLE
+            j3=i3+ind(3,j)
+            if(j3.le.0.or.j3.gt.n3) CYCLE
+            if(mask(j1,j2,j3).eq.0) CYCLE
+            wj=w(j)
+            z=(thi-th(j1,j2,j3))/sgi
+            z=z*z
+            if(z.ge.kval) CYCLE
+            wj=wj*min(1.d0,2.d0-2.d0*z/kval)
+            sw=sw+wj
+            sj=s(j1,j2,j3)
+            sws=sws+wj*sj
+            sws2=sws2+wj*sj*sj
+         END DO
+         ni(i) = sw
+         if(sw.gt.minni) THEN
+          sigman(i)= sqrt(max(0.d0,(sws2-sws/sw*sws)/(sw-1.d0)))
+C  using sigma not sigma^2
+         ELSE
+            sigman(i)=sgi
+         END IF
+         thn(i) = sws/sw
+      END DO
+C$OMP END DO NOWAIT
+C$OMP END PARALLEL
+C$OMP FLUSH(thn,ni,sigman)
+      RETURN
+      END
+
       subroutine awsvchi(y,th,ni,fns,mask,n1,n2,n3,ind,w,nw,lambda,
      1                    sigma,thn,sy)
 C   Takes noncentral Chi values in y
